﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Duplicates the top item on the INTEGER stack. Does not pop its argument
    /// (which, if it did, would negate the effect of the duplication!).
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class DuplicateExpression<T> : StatelessExpression
    {
        public void Eval(IStack<T> stack)
        {
            if (stack.Count == 0)
                return;

            stack.Push(stack.Top);
        }
    }

    public class IntegerDuplicateExpression : DuplicateExpression<long>
    {
        protected override string InitStringRepresentation() { return "INTEGER.DUP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack);
        }
    }

    public class FloatDuplicateExpression : DuplicateExpression<double>
    {
        protected override string InitStringRepresentation() { return "FLOAT.DUP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.FloatStack);
        }
    }

    public class BooleanDuplicateExpression : DuplicateExpression<bool>
    {
        protected override string InitStringRepresentation() { return "BOOLEAN.DUP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack);
        }
    }

    public class NameDuplicateExpression : DuplicateExpression<string>
    {
        protected override string InitStringRepresentation() { return "NAME.DUP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.NameStack);
        }
    }

    public class ExecDuplicateExpression : DuplicateExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "EXEC.DUP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.ExecStack);
        }
    }

    public class CodeDuplicateExpression : DuplicateExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "CODE.DUP"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.CodeStack);
        }
    }
}
