﻿using System;
using System.Linq;
using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    public class NameDefineXExecExpression : StatefullExpression
    {
        private readonly string name;
        public NameDefineXExecExpression(string name)
        {
            this.name = name;
        }

        public NameDefineXExecExpression(NameDefineXExecExpression origin, Cloner cloner) : base(origin, cloner)
        {
            name = origin.name;
        }

        public override void Eval(IInterpreter interpreter)
        {
            Expression expression;
            if (!interpreter.IsNameQuoteFlagSet &&
                interpreter.CustomExpressions.TryGetValue(name, out expression))
            {
                interpreter.ExecStack.Push(expression);
            }
            else
            {
                interpreter.NameStack.Push(name);
                interpreter.IsNameQuoteFlagSet = false;
            }
        }

        public override bool Equals(object obj)
        {
            if (!base.Equals(obj))
            {
                return false;
            }

            var other = obj as NameDefineXExecExpression;

            return name.Equals(other.name);
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        protected override string InitStringRepresentation()
        {
            return name;
        }

        protected override int InitId()
        {
            return name.GetHashCode();
        }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// Sets a flag indicating that the next name encountered will be pushed onto the NAME stack (and not have its associated value
    /// pushed onto the EXEC stack), regardless of whether or not it has a definition. Upon encountering such a name and pushing it
    /// onto the NAME stack the flag will be cleared (whether or not the pushed name had a definition).
    /// </summary>
    public class NameQuoteExpression : StatelessExpression
    {
        protected override string InitStringRepresentation()
        {
            return "NAME.QUOTE";
        }

        public override void Eval(IInterpreter interpreter)
        {
            interpreter.IsNameQuoteFlagSet = true;
        }
    }

    /// <summary>
    /// Pushes a randomly selected NAME that already has a definition.
    /// </summary>
    public class NameRandBoundNameExpression : StatelessExpression
    {
        private static IRandom rand = new FastRandom();

        protected override string InitStringRepresentation()
        {
            return "NAME.RANDBOUNDNAME";
        }

        public override void Eval(IInterpreter interpreter)
        {
            if (!interpreter.CustomExpressions.Any())
                return;

            var index = rand.Next(0, interpreter.CustomExpressions.Keys.Count - 1);
            var name = interpreter.CustomExpressions.Keys.ElementAt(index);

            interpreter.NameStack.Push(name);
        }
    }
}
