﻿using HeuristicLab.Algorithms.PushGP.Interpreter;
using HeuristicLab.Algorithms.PushGP.Stack;

namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    /// <summary>
    /// Pushes the stack depth onto the INTEGER stack (thereby increasing it!).
    /// </summary>
    /// <typeparam name="T">Stacktype</typeparam>
    public abstract class StackdepthExpression<T> : StatelessExpression
    {
        public void Eval(IStack<T> stack, IStack<long> integerStack, bool incremental = false)
        {
            var count = stack.Count;

            if (incremental)
            {
                count += 1;
            }

            integerStack.Push(count);
        }
    }

    public class IntegerStackdepthExpression : StackdepthExpression<long>
    {
        protected override string InitStringRepresentation() { return "INTEGER.STACKDEPTH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.IntegerStack, interpreter.IntegerStack, true);
        }
    }

    public class FloatStackdepthExpression : StackdepthExpression<double>
    {
        protected override string InitStringRepresentation() { return "FLOAT.STACKDEPTH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.FloatStack, interpreter.IntegerStack);
        }
    }

    public class BooleanStackdepthExpression : StackdepthExpression<bool>
    {
        protected override string InitStringRepresentation() { return "BOOLEAN.STACKDEPTH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.BooleanStack, interpreter.IntegerStack);
        }
    }

    public class NameStackdepthExpression : StackdepthExpression<string>
    {
        protected override string InitStringRepresentation() { return "NAME.STACKDEPTH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.NameStack, interpreter.IntegerStack);
        }
    }

    public class ExecStackdepthExpression : StackdepthExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "EXEC.STACKDEPTH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.ExecStack, interpreter.IntegerStack);
        }
    }

    public class CodeStackdepthExpression : StackdepthExpression<Expression>
    {
        protected override string InitStringRepresentation() { return "CODE.STACKDEPTH"; }

        public override void Eval(IInterpreter interpreter)
        {
            Eval(interpreter.CodeStack, interpreter.IntegerStack);
        }
    }
}
